#include "configfile.h"
#include <qfile.h>
#include <stdlib.h>
#include <sstream>


ConfigFile & TheConfigFile ( const QString & sourcefile )
{
	static ConfigFile configFile ( sourcefile );
	return configFile;
}


ConfigFile::ConfigFile ( const QString & sourcefile )
	: m_valid ( ! sourcefile.isEmpty ( ) )
	, m_smooth ( true )
	, m_newimageSize ( 640, 480 )
	, m_jpegQuality ( 90 )
	, m_ratioIndex ( 0 )
{
	m_valid = ParseFile ( sourcefile );
}


ConfigFile::~ConfigFile ( )
{
}


bool ConfigFile::IsValid ( ) const
{
	return m_valid;
}


bool ConfigFile::NewImageSmooth ( ) const
{
	return m_smooth;
}

const QSize & ConfigFile::NewImageSize ( ) const
{
	return m_newimageSize;
}

int ConfigFile::NewImageJpegQuality ( ) const
{
	return m_jpegQuality < 1 ? 1 : m_jpegQuality > 100 ? 100 : m_jpegQuality;
}

int ConfigFile::NewImageRatioIndex ( ) const
{
	return m_ratioIndex;
}


bool ConfigFile::ParseFile ( const QString & sourcefile )
{
	// Open the file
	QFile file ( sourcefile );
	file.open ( IO_ReadOnly );
	
	QDomDocument document;
	if ( ! document.setContent ( &file ) )
	{
		qWarning ( "Unable to parse XML document \"%s\"", sourcefile.ascii ( ) );
		return false;
	}

	// Get the new image settings
	m_newimageSize = QSize ( GetFieldValueAsInt ( m_newimageSize.width ( ), document, "newimage", "x" ),
			                 GetFieldValueAsInt ( m_newimageSize.height ( ), document, "newimage", "y" ) );
	m_smooth = GetFieldValueAsBool ( m_smooth, document, "newimage", "smooth" );
	m_jpegQuality = GetFieldValueAsInt ( m_jpegQuality, document, "newimage", "jpegquality" );

	const char * ratioStrings [] = { "ignore", "width", "height", "smart", 0 };
	m_ratioIndex = GetFieldValueAsIndex ( m_ratioIndex, ratioStrings, document, "newimage", "ratio" );

	
	// Finished, close the file
	file.close ( );
	return true;
}


QDomNode ConfigFile::FindNode ( const QDomNode & element, const QString & type, const QString & name ) const
{
	for ( QDomNode node ( element.firstChild ( ) ); ! node.isNull ( ); node = node.nextSibling ( ) )
		if ( node.nodeName ( ) == type && node.attributes ( ).namedItem ( "name" ).nodeValue ( ) == name )
			return node;
	return QDomNode ( );
}


QString ConfigFile::GetFieldValue ( const QDomDocument & document, const QString & section, const QString & field ) const
{
	QDomNode sectionNode ( FindNode ( document.documentElement ( ), "section", section ) );
	if ( ! sectionNode.isNull ( ) )
	{
		QDomNode fieldNode ( FindNode ( sectionNode, "field", field ) );
		if ( ! fieldNode.isNull ( ) && fieldNode.hasChildNodes ( ) )
		{
			fieldNode.normalize ( );
			if ( fieldNode.firstChild ( ).isText ( ) )
				return fieldNode.firstChild ( ).nodeValue ( ).stripWhiteSpace ( );
		}
	}

	std::ostringstream error;
	error << "Unable to find field \"" << field.ascii ( ) << "\" in section \"" << section.ascii ( ) << "\"";
	throw ConfigFile::Exception ( error.str ( ) );
}


int ConfigFile::GetFieldValueAsInt ( int defaultValue, const QDomDocument & document, const QString & section, const QString & field ) const
{
	try
	{
		QString value ( GetFieldValue ( document, section, field ) );
		int intValue ( atoi ( value.ascii ( ) ) );
		if ( intValue > 0 ) return intValue;

		std::ostringstream error;
		error << "Unable to parse value of " << section.ascii ( ) << "." << field.ascii ( ) << ", \"" << value.ascii ( ) << "\" as integer";
		throw ConfigFile::Exception ( error.str ( ) );
	}
	catch ( const ConfigFile::Exception & exception )
	{
		qWarning ( "Config Error - %s, using default of %d", exception.what ( ), defaultValue );
	}
	return defaultValue;
}


bool ConfigFile::GetFieldValueAsBool ( bool defaultValue, const QDomDocument & document, const QString & section, const QString & field ) const
{
	try
	{
		QString value ( GetFieldValue ( document, section, field ).lower ( ) );
		if ( value == "true" ) return true;
		if ( value == "false" ) return false;
		std::ostringstream error;
		error << "Unable to parse value of " << section.ascii ( ) << "." << field.ascii ( ) << ", \"" << value.ascii ( ) << "\" as bool [true|false]";
		throw ConfigFile::Exception ( error.str ( ) );
	}
	catch ( const ConfigFile::Exception & exception )
	{
		qWarning ( "Config Error - %s, using default of %s", exception.what ( ), defaultValue ? "true" : "false" );
	}
	return defaultValue;
}


int ConfigFile::GetFieldValueAsIndex ( int defaultValue, const char * list [], const QDomDocument & document, const QString & section, const QString & field ) const
{
	try
	{
		QString value ( GetFieldValue ( document, section, field ).lower ( ) );
		for ( int index ( 0 ); list [ index ]; ++index )
			if ( value == list [ index ] ) return index;
		std::ostringstream error;
		error << "Unable to parse value of " << section.ascii ( ) << "." << field.ascii ( ) << ", \"" << value.ascii ( ) << "\" as one of:";
		for ( const char * * element ( list ); *element; ++element ) error << " " << *element;
		throw ConfigFile::Exception ( error.str ( ) );
	}
	catch ( const ConfigFile::Exception & exception )
	{
		qWarning ( "Config Error - %s, using default of %s", exception.what ( ), defaultValue ? "true" : "false" );
	}
	return defaultValue;
}


//////////////////////////////////////////////////////////////////////////////
// Exception

ConfigFile::Exception::Exception ( )
{
}

ConfigFile::Exception::Exception ( const QString & message )
	: m_message ( message )
{
}

ConfigFile::Exception::Exception ( const std::string & message )
	: m_message ( message.c_str ( ) )
{
}

ConfigFile::Exception::Exception ( const ConfigFile::Exception & source )
	: std::exception ( )
	, m_message ( source.m_message )
{
}

ConfigFile::Exception::Exception ( const std::exception & source )
	: m_message ( source.what ( ) )
{
}

ConfigFile::Exception::~Exception ( ) throw ( )
{
}

const char * ConfigFile::Exception::what ( ) const throw ( )
{
	return m_message.ascii ( );
}

