#include "mainwindow.h"
#include "imageformats.h"
#include "mimefactory.h"
#include "configfile.h"
#include <qapplication.h>
#include <qstringlist.h>
#include <qmessagebox.h>
#include <qdir.h>
#include <stdlib.h>

// Used by the embedded image code
void qInitImages_birt ( );

// If no INSTALLPATH is provided default to the current directory
#ifndef INSTALLPATH
#	define INSTALLPATH "."
#endif


QString FindFile ( const QString & filename, const QString & exeFilename, const QString & envVar )
{
	// Build up a list of potential paths
	QStringList paths;
	paths.append ( filename );
	paths.append ( QFileInfo ( exeFilename ).dirPath ( ) + QDir::separator ( ) + filename );
	paths.append ( getenv ( envVar.ascii ( ) ) + ( QDir::separator ( ) + filename ) );
	paths.append ( INSTALLPATH + ( QDir::separator ( ) + filename ) );

	// Loop for the file
	for ( QStringList::const_iterator path ( paths.constBegin ( ) ); path != paths.constEnd ( ); ++path )
	{
		QString cleanPath ( QDir::cleanDirPath ( *path ) );
		if ( QFile::exists ( cleanPath ) )	
			return cleanPath;
	}

	return "";
}


void SetupMimeFactory ( const QString & filename, const QString & exeFilename, const QString & envVar )
{
	QString path ( FindFile ( filename, exeFilename, envVar ) );
	if ( ! path.isEmpty ( ) )
		TheMimeFactory ( path );
	else
		// Create an empty mime factory - the user will be warned later on in the
		// startup process
		TheMimeFactory ( );
}


void SetupConfigFile ( const QString & filename, const QString & exeFilename, const QString & envVar )
{
	QString path ( FindFile ( filename, exeFilename, envVar ) );
	if ( ! path.isEmpty ( ) )
		TheConfigFile ( path );
	else
		// Same as with the MimeFactory - the user will be warned later
		TheConfigFile ( );
}


void WarnIfNoHelp ( QWidget & window, const QString & filename, const QString & envVar )
{
	if ( ! TheMimeFactory ( ).IsValid ( ) )
		QMessageBox::warning ( &window, "Missing Help File", 
				               "Cannot find help file \"" + filename + "\". On some systems it may be\n" +
							   "necessary to run BIRT from its install directory (using the 'set working\n" +
							   "directory' option in your window manager) or to set the environmental\n" +
							   "variable \"" + envVar + "\" to the install path.\n\n" +
							   "Without this file the 'Help' button is disabled.", "Dismiss" );
}


void WarnIfNoConfig ( QWidget & window, const QString & filename, const QString & envVar )
{
	if ( ! TheConfigFile ( ).IsValid ( ) )
		QMessageBox::warning ( &window, "Missing Configuration File",
				               "Cannot find config file \"" + filename + "\". On some systems it may\n" +
							   "necessary to run BIRT from its install directory (using the 'set\n"
							   "working directory' option in your window manager) or to set the\n" +
							   "environmental variable \"" + envVar + "\" to the install path.\n\n" +
							   "With no configuration file BIRT will default to the hard-coded defaults.",
							   "Dismiss" );
}


void WarnIfImageFormatsUnavailable ( QWidget & window, const QString & commaSepTypes )
{
	// Create a list of image formats
	ImageFormats formats;
	QStringList types ( QStringList::split ( ',', commaSepTypes ) );
	QString errors;

	// If an image isn't in the list, add it to the errors - indented and newline terminated
	for ( QStringList::const_iterator type ( types.constBegin ( ) ); type != types.constEnd ( ); ++type )
		if ( ! formats.IsValidFormat ( *type ) )
			errors += "    - " + *type + "\n";

	if ( ! errors.isEmpty ( ) )
		QMessageBox::warning ( &window, "Image Formats Unsupported",
				               "Your Qt library lacks support for the following image formats :\n\n" + errors +
							   "\nBIRT will be unable to process images of this type. Check that the\n" +
							   "Qt library on this machine was configured correctly", "Dismiss" );
}


QPixmap GetEmbeddedIcon ( const QString & iconName )
{
	// Some compilers don't construct static objects properly - this forces the embedded image code 
	// to be correctly initialised
	qInitImages_birt ( );

	// Hopefully the embedded image should have been loaded in to the default mime factory
	return QPixmap::fromMimeSource ( iconName );
}


int main ( int argc, char * argv [ ] )
{
	const QString helpFile ( "birthelp.xml" ), configFile ( "birtconfig.xml" ), iconName ( "birticon.xpm" ), envVar ( "BIRTDIR" );

	// Setup the help file mime factory and the configuration file
	SetupMimeFactory ( helpFile, argv [ 0 ], envVar );
	SetupConfigFile ( configFile, argv [ 0 ], envVar );
	
	// Create the application and the primary window
	QApplication theApp ( argc, argv );

	MainWindowWidget window ( theApp );
	window.setIcon ( GetEmbeddedIcon ( iconName ) );
	
	// When the 'quit' button on the main window is clicked, quit the application
	QObject::connect ( &window, SIGNAL ( Quit ( ) ), &theApp, SLOT ( quit ( ) ) );
	
	// Display the window
	theApp.setMainWidget ( &window );
	window.show ( );

	// Throw any pending warnings
	WarnIfImageFormatsUnavailable ( window, "png,bmp,jpeg" );
	WarnIfNoHelp ( window, helpFile, envVar );
	WarnIfNoConfig ( window, configFile, envVar );

	return theApp.exec ( );
}
