#include "mainwindow.h"
#include "filechooserwidget.h"
#include "processingoptionswidget.h"
#include "outputoptionswidget.h"
#include "commandswidget.h"
#include "errordialog.h"
#include "helpdialog.h"
#include <qlayout.h>
#include <qmessagebox.h>
#include <qprogressdialog.h>
#include <qfileinfo.h>
#include <qimage.h>
#include <qdir.h>
#include <sys/types.h>
#include <sys/stat.h>


MainWindowWidget::MainWindowWidget ( QApplication & parent, const char * name, WFlags flags )
	: QWidget ( NULL, name, flags )
	, m_parent ( parent )
	, m_width ( 550 )
	, m_height ( 260 )
	, m_files ( NULL )
	, m_options ( NULL )
	, m_output ( NULL )
	, m_commands ( NULL )
	, m_help ( NULL )
	, m_error ( NULL )
{
	setCaption ( "BIRT" );
	
	// Create the layout
	QVBoxLayout * layout ( new QVBoxLayout ( this ) );
	layout->setResizeMode ( QLayout::Minimum );
	layout->setMargin ( 5 );
	layout->setSpacing ( 5 );

	// Create subwidgets
	m_files = new FileChooserWidget ( this );
	m_options = new ProcessingOptionsWidget ( this );
	m_output = new OutputOptionsWidget ( this );
	m_commands = new CommandsWidget ( this );

	// Make sure that the file widget is at least 512 pixels wide and 256 high, the other 
	// widgets can have the same width, but determine their own height
	m_files->setMinimumSize ( m_width, m_height );
	m_options->setMinimumWidth ( m_width );
	m_output->setMinimumWidth ( m_width );
	m_commands->setMinimumWidth ( m_width );

	// Attach widgets to the layout
	layout->addWidget ( m_files );
	layout->addWidget ( m_options );
	layout->addWidget ( m_output );
	layout->addWidget ( m_commands );
	
	// Setup signalling
	connect ( m_output, SIGNAL ( DirectoryChanged ( const QString & ) ), m_commands, SLOT ( NewOutputDir ( const QString & ) ) );
	connect ( m_commands, SIGNAL ( Quit ( ) ), this, SLOT ( QuitCommand ( ) ) );
	connect ( m_commands, SIGNAL ( Go ( ) ), this, SLOT ( GoCommand ( ) ) );
	connect ( m_commands, SIGNAL ( Help ( ) ), this, SLOT ( HelpCommand ( ) ) );
	connect ( m_files, SIGNAL ( SelectionChanged ( unsigned int ) ), m_commands, SLOT ( FileSelectionChanged ( unsigned int ) ) );
}


void MainWindowWidget::QuitCommand ( )
{
	// Ask the use whether they're sure they want to quit
	if ( QMessageBox::question ( this, "Quit ?", "Exit Application ?", QMessageBox::No, QMessageBox::Yes ) == QMessageBox::Yes )
		emit ( Quit ( ) );
}


bool MainWindowWidget::isJPEG ( const QString & format ) const
{
	return format.stripWhiteSpace ( ).lower ( ) == "jpeg";
}


void MainWindowWidget::GoCommand ( )
{
	// Get the information required to do the processing
	QStringList files ( static_cast < FileChooserWidget * > ( m_files )->GetFiles ( ) );
	unsigned int numFiles ( files.size ( ) );

	OutputOptionsWidget & output ( *( static_cast < OutputOptionsWidget * > ( m_output ) ) );
	QString outputDirectory ( output.GetOutputDirectory ( ) );
	QString outputPrefix ( output.GetOutputPrefix ( ) );
	QString outputSuffix ( output.GetOutputSuffix ( ) );

	ProcessingOptionsWidget & options ( *( static_cast < ProcessingOptionsWidget * > ( m_options ) ) );
	QSize outputSize ( options.GetOutputSize ( ) );
	bool smoothResize ( options.IsQualityOptionSet ( "smooth" ) );
	unsigned int jpegQuality ( options.GetJPEGQuality ( ) );

	// Keep track of any errors
	QStringList errors;
	
	// Create a process dialog box
	QProgressDialog progress ( "Processing image files", "Cancel", numFiles, this, NULL, true );
	progress.setMinimumDuration ( 0 );

	// Process each file in turn
	for ( unsigned int index ( 0 ); index < numFiles; ++index )
	{
		// Update the display and catch the cancel button
		progress.setProgress ( index );
		m_parent.processEvents ( );
		if ( progress.wasCanceled ( ) )
			break;

		// Get the new file name
		const QString & oldFilename ( files [ index ] );
		QString newFilename ( GenerateNewFilename ( oldFilename, outputDirectory, outputPrefix, outputSuffix ) );

		// Load the original image file
		const char * imageFormat ( QImage::imageFormat ( oldFilename ) );
		QImage image ( oldFilename );
        
		if ( image.isNull ( ) )
		{
			if ( QFile::exists ( oldFilename ) )
			{
				if ( imageFormat )
					errors.append ( "Image file \"" + oldFilename + "\" was of a recognised format, " + QString ( imageFormat ) + ", but was unloadable" );
				else
					errors.append ( "Image file \"" + oldFilename + "\" is not of a usable format" );
			}
			else
				errors.append ( "Image file \"" + oldFilename + "\" cannot be read" );
			continue;
		}
		
		// Load a large image could have taken a while, so be polite and process any pending events
		m_parent.processEvents ( );
		if ( progress.wasCanceled ( ) )
			break;

		// Resize the image
		QImage newImage;
		if ( options.RatioIgnore ( ) )
			newImage = ResizeImage ( image, outputSize, smoothResize, false );
		else
			newImage = ResizeImage ( image, CalculateNewSize ( image.size ( ), outputSize, options.RatioWidth ( ), options.RatioSmart ( ) ), smoothResize, false );

		// Again, we don't want to be a system hog
		m_parent.processEvents ( );
		if ( progress.wasCanceled ( ) )
			break;

		// Save the image, but only if the destination file doesn't exist
		//
		// NB - For some reason the QFileInfo.exists method ignores both setCaching ( false ) and
		//      refresh ( ). Thus any file that is seen as existing will continue to do so in the
		//      eyes of QFileInfo for the lifetime of the process, even if the file has been
		//      removed in the meantime (outside of the process). This makes QFileInfo pretty
		//      useless for this purpose so, until a workround can be found, it has been replaced
		//      by a call to stat.
		//      VS   2003/7/7   gcc 2.95.5   qt 3.2.0b2   Solaris 2.8
		struct stat statBuffer;
		if ( stat ( newFilename.ascii ( ), &statBuffer ) != 0 )
			newImage.save ( newFilename, QImage::imageFormat ( oldFilename ), isJPEG ( imageFormat ) ? jpegQuality : 0 );
		else
			errors.append ( "Cannot create file \"" + newFilename + "\", already exists" );
	}

	// Finished
	progress.setProgress ( numFiles );

	// Success?
	if ( ! errors.empty ( ) )
	{
		if ( QMessageBox::warning ( this, "Errors encountered",
				                    QString ( "%1 of the %2 files were not sucessfully processed, would you like to see a report ?" )
									          .arg ( errors.size ( ) ).arg ( numFiles ),
									QMessageBox::No, QMessageBox::Yes ) == QMessageBox::Yes )
		{
			// Remove the existing error dialog (if any)
			if ( m_error )
				delete m_error;
			
			// Open the error dialog (non-modal)
			m_error = new ErrorDialog ( errors, this );
			m_error->show ( );

			connect ( m_error, SIGNAL ( Closing ( ) ), this, SLOT ( ErrorClosed ( ) ) );
		}
	}
	else
		QMessageBox::information ( this, "Processing Successful", QString ( "Successfully processed %1 files" ).arg ( numFiles ),
				                   QMessageBox::Yes );
}


void MainWindowWidget::HelpCommand ( )
{
	// If the help dialog exists - then ensure it is shown (clicking the close
	// window button on the title bar simply hides a window); otherwise create
	// a new dialog and display that.
	if ( m_help )
		m_help->show ( );
	else
	{
		m_help = new HelpDialog;
		connect ( m_help, SIGNAL ( Closing ( ) ), this, SLOT ( HelpClosed ( ) ) );
		m_help->show ( );
	}
}


void MainWindowWidget::HelpClosed ( )
{
	if ( m_help )
	{
		delete m_help;
		m_help = NULL;
	}
}


void MainWindowWidget::ErrorClosed ( )
{
	if ( m_error )
	{
		delete m_error;
		m_error = NULL;
	}
}


QImage MainWindowWidget::ResizeImage ( const QImage & image, const QSize & size, bool smooth, bool ratio ) const
{
	QImage::ScaleMode type ( ratio ? QImage::ScaleMin : QImage::ScaleFree );

	return smooth ? image.smoothScale ( size, type ) : image.scale ( size, type );
}


QString MainWindowWidget::GenerateNewFilename ( const QString & filename, const QString & dir, const QString & prefix, const QString & suffix ) const
{
	QFileInfo fileInfo ( filename );

	// NB - The QFileInfo::baseName and QFileInfo::extension calls are currently set to treat the final period as the
	//      extension delimiter. By passing them false the first period can be used to delimit extensions.
	return QDir::cleanDirPath ( dir + "/" + prefix + fileInfo.baseName ( true ) + suffix + "." + fileInfo.extension ( true ) );
}
		

QSize MainWindowWidget::CalculateNewSize ( const QSize & currentSize, const QSize & desiredSize, bool keepWidth, bool smart ) const
{
	double currentWidth ( currentSize.width ( ) ), currentHeight( currentSize.height ( ) );
	double desiredWidth ( desiredSize.width ( ) ), desiredHeight( desiredSize.height ( ) );
	
    if ( smart )
    {
		if ( currentWidth > currentHeight )
        {
			// Source is landscape
            if ( desiredWidth > desiredHeight )
				return QSize ( static_cast < size_t> ( desiredWidth ) , static_cast < size_t > ( desiredHeight ) ); // Desired size was landscape
            else
        		return QSize ( static_cast < size_t> ( desiredHeight ) , static_cast < size_t > ( desiredWidth ) );	// Desired size was portrait
        }
        else
        {
			// Source is portrait
			if ( desiredWidth > desiredHeight )
				return QSize ( static_cast < size_t> ( desiredHeight ) , static_cast < size_t > ( desiredWidth ) ); // Desired size was landscape
			else
				return QSize ( static_cast < size_t> ( desiredWidth ) , static_cast < size_t > ( desiredHeight ) ); // Desired size was portrait
        }
    }
    else
    {
		if ( keepWidth )
			return QSize ( static_cast < size_t> ( desiredWidth ) , static_cast < size_t > ( ( desiredWidth / currentWidth ) * currentHeight ) );
        else
            return QSize ( static_cast < size_t > ( ( desiredHeight / currentHeight ) * currentWidth ), static_cast < size_t > ( desiredHeight ) );
    }
}
