#include "mimefactory.h"
#include <qfile.h>
#include <qregexp.h>
#include "imageformats.h"
#include "uudecoder.h"


MimeFactory & TheMimeFactory ( const QString & sourcefile )
{
	static MimeFactory mimeFactory ( sourcefile );
	return mimeFactory;
}


MimeFactory::MimeFactory ( const QString & sourcefile )
	: m_valid ( ! sourcefile.isEmpty ( ) )
{
	ParseFile ( sourcefile );
}


MimeFactory::~MimeFactory ( ) 
{
}


bool MimeFactory::IsValid ( ) const
{
	return m_valid;
}


QStringList MimeFactory::GetPageNames ( ) const
{
	return GetNames ( "page" );
}


QStringList MimeFactory::GetImageNames ( ) const
{
	return GetNames ( "image" );
}


QStringList MimeFactory::GetNames ( const QString & type ) const
{
	QStringList names;
	
	QDomElement root ( m_document.documentElement ( ) );
	for ( QDomNode node ( root.firstChild ( ) ); ! node.isNull ( ); node = node.nextSibling ( ) )
		if ( node.nodeName ( ) == type )
			names.append ( QDomAttr ( node.toElement ( ).attributeNode ( "name" ) ).value ( ) );

	return names;
}


QDomElement MimeFactory::GetElement ( const QString & type, const QString & name ) const
{
	QDomElement root ( m_document.documentElement ( ) );
	for ( QDomNode node ( root.firstChild ( ) ); ! node.isNull ( ); node = node.nextSibling ( ) )
		if ( node.nodeName ( ) == type )
		{
			QDomElement element ( node.toElement ( ) );
			QDomAttr nameAttr ( element.attributeNode ( "name" ) );

			if ( nameAttr.value ( ) == name )
				return element;
		}
	
	return QDomElement ( );
}


QString MimeFactory::GetText ( const QString & name ) const
{
	QDomElement element ( GetElement ( "page", name ) );
	return NodeToString ( element );
}


QImage MimeFactory::GetImage ( const QString & name ) const
{
	QDomElement element ( GetElement ( "image", name ) );

	// Get the image as uuencoded text, and remove any indentation
	QString text ( element.text ( ) );
	text.remove ( QRegExp ( "\t+" ) );
	text.remove ( QRegExp ( "^[\r\n\r ]+" ) );

	// Decode the image
	QByteArray outputString ( UUDecoder::Decode ( text ) );

	// Attempt to create the image object
	QImage newImage;
	if ( newImage.loadFromData ( outputString ) )
		return newImage;
	else
	{
		qWarning ( "Unable to load image \"%s\"", name.ascii ( ) );
		return QImage ( );
	}
}


bool MimeFactory::LoadFile ( QDomDocument & document, const QString & sourcefile ) const
{
	QFile file ( sourcefile );
	file.open ( IO_ReadOnly );

	if ( ! document.setContent ( &file ) )
	{
		qWarning ( "Unable to parse XML document \"%s\"", sourcefile.ascii ( ) );
		return false;
	}

	file.close ( );
	return true;
}


bool MimeFactory::ParseFile ( const QString & sourcefile )
{
	return LoadFile ( m_document, sourcefile ) ? true : ( m_valid = false );
}


QString MimeFactory::NodeToString ( QDomNode & node ) const
{
	QString text;
	QTextStream textStream ( &text, IO_WriteOnly );

	node.normalize ( );
	node.firstChild ( ).save ( textStream, 1 );

	ProcessString ( text );
	
	return text;
}
		

void MimeFactory::ProcessString ( QString & string ) const
{
	// Create objects required for substitution
	ImageFormats imageFormats;
	
	if ( QRegExp ( "\%\%[^\%]+\%\%" ).search ( string ) >= 0 )
	{
		// Replace tags with various values
		string.replace ( "%%BUILDTIME%%", QString ( "Built %1 %2" ).arg ( __DATE__ ).arg (  __TIME__ ) );
		string.replace ( "%%IMAGEFORMATS%%", imageFormats.GetImageFormats ( ).join ( " " ) );
		string.replace ( "%%QTVERSION%%", QString ( "Qt %1" ).arg ( qVersion ( ) ) );
	}
}
